function R = RS(z, A, idx, kappa, R, itrMax, tolerance)
disp('================== RS_RLS ==================');

m = length(z);
N = length(R);

idxp = idx;
idxp(idxp(:,1)==0,1) = N+1;
idxp(idxp(:,2)==0,2) = N+1;
z_ = zeros(m,9);
for i = 1:m
    z_(i,:) = reshape(z{i}',1,9);
end
R_ = zeros(N,9);
for i = 1:N
    R_(i,:) = reshape(R{i}',1,9);
end

%% Iterative Algorithm
Ak = diag(sqrt(kappa))*A;
Ak = sparse(Ak);

itr = 0;
tic
while 1
    itr = itr + 1;
    
    R_(N+1,:) = [1 0 0 0 1 0 0 0 1];
    Ri = R_(idxp(:,1),:);
    Rj = R_(idxp(:,2),:);
    R_ = R_(1:N,:);
    
    % r -> Ri    R -> Rj
    r1 = Ri(:,1); r2 = Ri(:,2); r3 = Ri(:,3);
    r4 = Ri(:,4); r5 = Ri(:,5); r6 = Ri(:,6);
    r7 = Ri(:,7); r8 = Ri(:,8); r9 = Ri(:,9);
    R1 = Rj(:,1); R2 = Rj(:,2); R3 = Rj(:,3);
    R4 = Rj(:,4); R5 = Rj(:,5); R6 = Rj(:,6);
    R7 = Rj(:,7); R8 = Rj(:,8); R9 = Rj(:,9);
    z1 = z_(:,1); z2 = z_(:,2); z3 = z_(:,3);
    z4 = z_(:,4); z5 = z_(:,5); z6 = z_(:,6);
    z7 = z_(:,7); z8 = z_(:,8); z9 = z_(:,9);
    
    SS = [
    r1.*(R1.*z1 + R4.*z4 + R7.*z7) + r4.*(R1.*z2 + R4.*z5 + R7.*z8) + r7.*(R1.*z3 + R4.*z6 + R7.*z9),...
    r2.*(R1.*z1 + R4.*z4 + R7.*z7) + r5.*(R1.*z2 + R4.*z5 + R7.*z8) + r8.*(R1.*z3 + R4.*z6 + R7.*z9),...
    r3.*(R1.*z1 + R4.*z4 + R7.*z7) + r6.*(R1.*z2 + R4.*z5 + R7.*z8) + r9.*(R1.*z3 + R4.*z6 + R7.*z9),...
    r1.*(R2.*z1 + R5.*z4 + R8.*z7) + r4.*(R2.*z2 + R5.*z5 + R8.*z8) + r7.*(R2.*z3 + R5.*z6 + R8.*z9),...
    r2.*(R2.*z1 + R5.*z4 + R8.*z7) + r5.*(R2.*z2 + R5.*z5 + R8.*z8) + r8.*(R2.*z3 + R5.*z6 + R8.*z9),...
    r3.*(R2.*z1 + R5.*z4 + R8.*z7) + r6.*(R2.*z2 + R5.*z5 + R8.*z8) + r9.*(R2.*z3 + R5.*z6 + R8.*z9),...
    r1.*(R3.*z1 + R6.*z4 + R9.*z7) + r4.*(R3.*z2 + R6.*z5 + R9.*z8) + r7.*(R3.*z3 + R6.*z6 + R9.*z9),...
    r2.*(R3.*z1 + R6.*z4 + R9.*z7) + r5.*(R3.*z2 + R6.*z5 + R9.*z8) + r8.*(R3.*z3 + R6.*z6 + R9.*z9),...
    r3.*(R3.*z1 + R6.*z4 + R9.*z7) + r6.*(R3.*z2 + R6.*z5 + R9.*z8) + r9.*(R3.*z3 + R6.*z6 + R9.*z9),...
    ];
    S_ = -[SS(:,8)-SS(:,6) SS(:,3)-SS(:,7) SS(:,4)-SS(:,2)] / 2;    

    Bk = zeros(m,3);
    sk = sqrt(kappa);
    Bk(:,1) = S_(:,1).*sk;
    Bk(:,2) = S_(:,2).*sk;
    Bk(:,3) = S_(:,3).*sk;
    Bk = sparse(Bk);
    
    delta = (Ak'*Ak)\(Ak'*Bk);
    %--------------- Update Rotations -----------------

    delta = full(delta);
    Sd = sqrt(sum(delta.^2,2));
    idxsd = Sd>1;
    deltan = delta./([Sd Sd Sd]);
    delta(idxsd,:) = deltan(idxsd,:);
    Del = sum(delta.^2,2);
    beta = real((1 - cos(asin(sqrt(Del))))./Del);

%  r3*(d2 + b*d1*d3) - r2*(d3 - b*d1*d2) - r1*(b*(d2^2 + d3^2) - 1)
    R_1 = ...
    R_(:,3).*(delta(:,2)+beta.*delta(:,1).*delta(:,3)) -...
    R_(:,2).*(delta(:,3)-beta.*delta(:,1).*delta(:,2)) -...
    R_(:,1).*(beta.*(delta(:,2).^2+delta(:,3).^2) - 1);
%  r6*(d2 + b*d1*d3) - r5*(d3 - b*d1*d2) - r4*(b*(d2^2 + d3^2) - 1)
    R_4 = ...
    R_(:,6).*(delta(:,2)+beta.*delta(:,1).*delta(:,3)) -...
    R_(:,5).*(delta(:,3)-beta.*delta(:,1).*delta(:,2)) -...
    R_(:,4).*(beta.*(delta(:,2).^2+delta(:,3).^2) - 1);
%  r9*(d2 + b*d1*d3) - r8*(d3 - b*d1*d2) - r7*(b*(d2^2 + d3^2) - 1)
    R_7 = ...
    R_(:,9).*(delta(:,2)+beta.*delta(:,1).*delta(:,3)) -...
    R_(:,8).*(delta(:,3)-beta.*delta(:,1).*delta(:,2)) -...
    R_(:,7).*(beta.*(delta(:,2).^2+delta(:,3).^2) - 1);
%  r1*(d3 + b*d1*d2) - r3*(d1 - b*d2*d3) - r2*(b*(d1^2 + d3^2) - 1)
    R_2 = ...
    R_(:,1).*(delta(:,3)+beta.*delta(:,1).*delta(:,2)) -...
    R_(:,3).*(delta(:,1)-beta.*delta(:,2).*delta(:,3)) -...
    R_(:,2).*(beta.*(delta(:,1).^2+delta(:,3).^2) - 1);
%  r4*(d3 + b*d1*d2) - r6*(d1 - b*d2*d3) - r5*(b*(d1^2 + d3^2) - 1)
    R_5 = ...
    R_(:,4).*(delta(:,3)+beta.*delta(:,1).*delta(:,2)) -...
    R_(:,6).*(delta(:,1)-beta.*delta(:,2).*delta(:,3)) -...
    R_(:,5).*(beta.*(delta(:,1).^2+delta(:,3).^2) - 1);
%  r7*(d3 + b*d1*d2) - r9*(d1 - b*d2*d3) - r8*(b*(d1^2 + d3^2) - 1)
    R_8 = ...
    R_(:,7).*(delta(:,3)+beta.*delta(:,1).*delta(:,2)) -...
    R_(:,9).*(delta(:,1)-beta.*delta(:,2).*delta(:,3)) -...
    R_(:,8).*(beta.*(delta(:,1).^2+delta(:,3).^2) - 1);
%  r2*(d1 + b*d2*d3) - r1*(d2 - b*d1*d3) - r3*(b*(d1^2 + d2^2) - 1)
    R_3 = ...
    R_(:,2).*(delta(:,1)+beta.*delta(:,2).*delta(:,3)) -...
    R_(:,1).*(delta(:,2)-beta.*delta(:,1).*delta(:,3)) -...
    R_(:,3).*(beta.*(delta(:,1).^2+delta(:,2).^2) - 1);
%  r5*(d1 + b*d2*d3) - r4*(d2 - b*d1*d3) - r6*(b*(d1^2 + d2^2) - 1)
    R_6 = ...
    R_(:,5).*(delta(:,1)+beta.*delta(:,2).*delta(:,3)) -...
    R_(:,4).*(delta(:,2)-beta.*delta(:,1).*delta(:,3)) -...
    R_(:,6).*(beta.*(delta(:,1).^2+delta(:,2).^2) - 1);
%  r8*(d1 + b*d2*d3) - r7*(d2 - b*d1*d3) - r9*(b*(d1^2 + d2^2) - 1)
    R_9 = ...
    R_(:,8).*(delta(:,1)+beta.*delta(:,2).*delta(:,3)) -...
    R_(:,7).*(delta(:,2)-beta.*delta(:,1).*delta(:,3)) -...
    R_(:,9).*(beta.*(delta(:,1).^2+delta(:,2).^2) - 1);
        
    R_ = [R_1 R_2 R_3 R_4 R_5 R_6 R_7 R_8 R_9];

    f = 2*sum(sum((Ak*delta-Bk).^2));
    msd = sqrt(max(sum(delta.*delta,2)));
%     fprintf('max ||delta|| = %4.2e\n', msd);
    if itr > 1
%         fprintf('df/f = %4.2e\n', full((f_old-f)/f));
        if (f_old-f)/f < 1e-7
%             break;
        end
    end
    if ~(msd > tolerance && itr < itrMax)
        break;
    end
    f_old = f;
end
toc
for i = 1:N
    R{i} = reshape(R_(i,:),3,3)';
end

