%% Load File
% g2ofilename = 'data/cubicle.g2o';
format short g
g2ofilename = 'data/road.g2o';

load road_gt

[z, d, idx, tau, kappa, N, A, anchid] = loadFile(g2ofilename);
m = length(z);

%% Chordal Initialization
R = ChordInit(z, d, idx, kappa, N);

%% Algorithm
tolerance = 1e-7;
itrMax = 100;

R = RS(z, A, idx, kappa, R, itrMax, tolerance);
P = Pos_LS(d,R,A,tau',idx);
Sm = SE(P,R,d,z,idx,kappa,tau);
fprintf('\nRS Cost: %6.3e\n',Sm);

[R, P] = PS(z, d, A, idx, tau, kappa, R, itrMax, tolerance);
Sm = SE(P,R,d,z,idx,kappa,tau);
fprintf('PS Cost: %6.2f\n',Sm);
ATE = sum(sqrt(sum((Ph-Pr).^2,2)));
fprintf('ATE Value: %6.0f\n\n',ATE);

R2 = cell(100,1);
for i = 1:300
    R2{i} = angle2dcm(i*pi/180+pi/3,0,0);
end
[R2, P2] = PS(z, d, A, idx, tau, kappa, R2, itrMax, tolerance);
Sm = SE(P2,R2,d,z,idx,kappa,tau);
fprintf('PS Cost: %6.2f\n',Sm);

ATE2 = sum(sqrt(sum((Ph2-Pr).^2,2)));
fprintf('ATE Value: %6.0f\n',ATE2);

%% Plot
% plt(P,R,idx,anchid)
plot(P(:,1),P(:,2),'b','linewidth',1);
axis equal
grid on
hold on
plot(P2(:,1),P2(:,2),'r','linewidth',1);
plot(Pr(:,1),Pr(:,2),'color',[0 .8 0],'linewidth',1);

lgd = legend({'Global Optimum','Local Optimum','Ground Truth'});
lgd.Position(1:2) = [.6 .4];
ylim([-80 20])